import time
import logging
import click

from gpiozero import LED
from gpiozero.exc import BadPinFactory

# From current package
import inputdevice as idev

logger = logging.getLogger(__name__)

BOARDS_CONFIG = {
    "v2.0": {
        0: { # Port 0 (i.e connector J2)
            "down" : 4,
            "up"   : 17,
            "left" : 3,
            "right": 22,
            "fire" : 27,
        },
        1: {
            "down" : 0,
            "up"   : 5,
            "left" : 11,
            "right": 13,
            "fire" : 6,
        },
    },
    "v2.1": {
        0: { # Port 1 (i.e connector J3)
            "down" : 27,
            "up"   : 22,
            "left" : 17,
            "right": 4,
            "fire" : 10,
        },
        1: {
            "down" : 6,
            "up"   : 13,
            "left" : 5,
            "right": 0,
            "fire" : 19,
        },
    },
}

def rpi_init(board_version, port_id):
    pins = BOARDS_CONFIG[board_version][port_id]
    return {k:LED(v) for k,v in pins.items()}

def process_input_events(input_device, board_signals):
    dev = idev.InputDevice(input_device)
    logger.info(f"Opened device: {input_device}")
    while True:
        _, _, ev_type, ev_code, ev_value = dev.get_event()
        if  ev_type == idev.EV_ABS:

            if   ev_code == idev.ABS_HAT0X:
                logger.info(f"HAT0X move: {ev_value}")
                if   ev_value == -1: board_signals["left"].on()
                elif ev_value ==  1: board_signals["right"].on()
                else: # ev_value == 0 when releasing a button
                    board_signals["left"].off()
                    board_signals["right"].off()

            elif ev_code == idev.ABS_HAT0Y:
                logger.info(f"HAT0Y move: {ev_value}")
                if   ev_value == -1: board_signals["up"].on()
                elif ev_value ==  1: board_signals["down"].on()
                else: # ev_value == 0 when releasing a button
                    board_signals["down"].off()
                    board_signals["up"].off()

            elif ev_code == idev.ABS_X:
                if   ev_value <= 64 :
                    logger.info(f"X (analog left) move: {ev_value}")
                    board_signals["left"].on()
                    board_signals["right"].off()
                elif ev_value >= 192:
                    logger.info(f"X (analog left) move: {ev_value}")
                    board_signals["left"].off()
                    board_signals["right"].on()
                else: # 64 < ev_value < 192
                    board_signals["left"].off()
                    board_signals["right"].off()

            elif ev_code == idev.ABS_Y:
                if   ev_value <= 64 :
                    logger.info(f"Y (analog left) move: {ev_value}")
                    board_signals["up"].on()
                    board_signals["down"].off()
                elif ev_value >= 192:
                    logger.info(f"Y (analog left) move: {ev_value}")
                    board_signals["up"].off()
                    board_signals["down"].on()
                else: # 64 < ev_value < 192
                    board_signals["up"].off()
                    board_signals["down"].off()

        elif ev_type == idev.EV_KEY and ev_code == idev.BTN_SOUTH:
                logger.info(f"FIRE button: {ev_value}")
                if ev_value == 1: board_signals["fire"].on()
                else: board_signals["fire"].off()

@click.command()
@click.option("--board",  "-b", default="v2.1", type=click.Choice(['v2.0', 'v2.1']), help="Board revision.", show_default=True)
@click.option("--device", "-d", default="/dev/input/event0", help="Input device to use.", show_default=True)
@click.option("--port",   "-p", default='1', type=click.Choice(['0', '1']), help="Board/Atari ST port to connect the joystick to.", show_default=True)
@click.option("--debug/--no-debug", help="Display debugging information.", show_default=True)
def main(device, board, port, debug):
    """Send joystick/gamepad events to an Atari ST connected to the
    RetroDevEm board.  Usage example: atarist-joystick --board v2.0
    --device /dev/input/event0 --port 0

    """
    if debug:
        logging.basicConfig(level=logging.DEBUG)
    try:
        signals = rpi_init(board, int(port))
        while True:
            try:
                process_input_events(device, signals)
            except OSError as e:
                # Sometimes gamepads get temporarily disconnected with crap cables
                logger.warning(f"Error getting event: {e}")
            time.sleep(0.2)
    except BadPinFactory as e:
        logger.error(f"Failed to initialize GPIO pins: {e}")
        logger.error( "This program expects to be running on a Raspberry Pi. Is that the case ?")

if __name__ == "__main__":
    main()



import time
import logging
import click

from gpiozero import LED
from gpiozero.exc import BadPinFactory

from . import inputdevice as idev

logger = logging.getLogger(__name__)

BOARDS_CONFIG = {
    "v2.0": {
        0: { # Port 0 (i.e connector J2)
            "XA" : 4,
            "XB" : 17,
            "YA" : 3,
            "YB" : 22,
            "LB" : 27, # Left mouse button
            "RB" : 10, # Right mouse button
        },
        1: { # Port 1 (i.e connector J3)
            "XA" : 0,
            "XB" : 5,
            "YA" : 11,
            "YB" : 13,
            "LB" : 6,
            "RB" : 19,
        },
    },
    "v2.1": {
        0: {
            "XA" : 27,
            "XB" : 22,
            "YA" : 17,
            "YB" : 4,
            "LB" : 10,
            "RB" : 2,
        },
        1: {
            "XA" : 6,
            "XB" : 13,
            "YA" : 5,
            "YB" : 0,
            "LB" : 19,
            "RB" : 9,
        },
    },
}

# Mouse is polled at 62.5 Hz (obtained using evtest)
# Atari ST screen vertical freq in 50 Hz
REFRESH_PERIOD = 1 / 80
MAX_TICK_PERIOD = REFRESH_PERIOD / 2
MIN_TICK_PERIOD = 1 / 2000 # 2 KHz
TICK_PERIOD_DECAY = 1.01
STATS_PERIOD = 0.5 # seconds

A_SIGNAL = (0, 1, 1, 0)
B_SIGNAL = (0, 0, 1, 1)

def rpi_init(board_version, port_id):
    pins = BOARDS_CONFIG[board_version][port_id]
    return {k:LED(v) for k,v in pins.items()}

class StMouse:
    def __init__(self, board_version, port, xy_scale):
        self.xy_scale = xy_scale
        self.x_state = 0
        self.y_state = 0
        self.x_delta = 0
        self.y_delta = 0
        self.tick_period = MAX_TICK_PERIOD
        self.worst_delay = 0
        # Initialize and turn off every line
        self.signals = rpi_init(board_version, port)
        for sig in self.signals.values():
            sig.off()

    def x_step(self, dir):
        """dir can be 1 for right or -1 for left"""
        self.x_state = (self.x_state + dir) % 4
        if A_SIGNAL[self.x_state]: self.signals["XA"].on()
        else: self.signals["XA"].off()
        if B_SIGNAL[self.x_state]: self.signals["XB"].on()
        else: self.signals["XB"].off()

    def y_step(self, dir):
        self.y_state = (self.y_state + dir) % 4
        if A_SIGNAL[self.y_state]: self.signals["YB"].on()
        else: self.signals["YB"].off()
        if B_SIGNAL[self.y_state]: self.signals["YA"].on()
        else: self.signals["YA"].off()

    def btn_left(self, val):
        if val: self.signals["LB"].on()
        else  : self.signals["LB"].off()

    def btn_right(self, val):
        if val: self.signals["RB"].on()
        else  : self.signals["RB"].off()

    def x_move(self, val):
        self.x_delta += val

    def y_move(self, val):
        self.y_delta += val

    def update_tick_period(self):
        delta_steps = max(abs(self.x_delta), abs(self.y_delta)) / self.xy_scale
        if delta_steps >= 1:
            self.tick_period = min( self.tick_period, REFRESH_PERIOD / delta_steps ) # Only faster
            self.tick_period = max( self.tick_period, MIN_TICK_PERIOD )
            self.tick_period = min( self.tick_period, MAX_TICK_PERIOD )

    def update_worst_delay(self, ev_time):
        delta_steps = max(abs(self.x_delta), abs(self.y_delta)) / self.xy_scale
        if delta_steps >= 1: # We can't tell if we don't have any step in queue
            evt_delay = time.time() - ev_time
            sig_delay = evt_delay + delta_steps * self.tick_period
            self.worst_delay = max(self.worst_delay, sig_delay)

    def decay_tick_period(self):
        delta_steps = max(abs(self.x_delta), abs(self.y_delta)) / self.xy_scale
        if delta_steps < 1:
            self.tick_period = self.tick_period * TICK_PERIOD_DECAY
            self.tick_period = min(self.tick_period, MAX_TICK_PERIOD)

    def get_tick_period(self):
        return self.tick_period

    def signals_tick(self):
        # Move mouse pointer
        if abs(self.x_delta) >= self.xy_scale:
            # increment by (+/-) self.xy_scale
            inc = self.xy_scale * self.x_delta // abs(self.x_delta)
            self.x_delta -= inc # Works for positive and negative alike
            self.x_step(inc // self.xy_scale) # Step is by +/- 1
        if abs(self.y_delta) >= self.xy_scale:
            inc = self.xy_scale * self.y_delta // abs(self.y_delta)
            self.y_delta -= inc
            self.y_step(inc // self.xy_scale)
        self.decay_tick_period()

    def process_events(self, events):
        for ev_sec, ev_us, ev_type, ev_code, ev_value in events:
            if ev_type == idev.EV_REL:
                if   ev_code == idev.REL_X: self.x_move(ev_value)
                elif ev_code == idev.REL_Y: self.y_move(ev_value)
            elif ev_type == idev.EV_KEY:
                if   ev_code == idev.BTN_LEFT : self.btn_left(ev_value)
                elif ev_code == idev.BTN_RIGHT: self.btn_right(ev_value)
            self.update_tick_period()
            self.update_worst_delay(ev_sec + ev_us*1e-6)

    def display_stats(self):
        tick_us  = int(self.get_tick_period() * 1e6)
        delay_ms = int(self.worst_delay * 1e3)
        stats = f"Tick: {tick_us} us"
        if delay_ms: stats += f" - Delay: {delay_ms} ms"
        self.worst_delay = 0
        logger.info(stats)

def process_input_events(input_device, st_mouse):
    dev = idev.InputDevice(device=input_device, blocking=False)

    next_tick = time.monotonic() # time.monotonic is more accurate than time.time
    next_stat = next_tick
    while True:
        # Retrieve every event in the queue
        event_l = []
        event = dev.get_event() # Non blocking
        while event:
            event_l.append(event)
            event = dev.get_event()
        st_mouse.process_events(event_l)

        st_mouse.signals_tick()
        next_tick += st_mouse.get_tick_period()
        # Display some statistics
        if time.monotonic() > next_stat:
            st_mouse.display_stats()
            next_stat += STATS_PERIOD
        try:
            time.sleep( next_tick - time.monotonic() )
        except ValueError:
            pass

@click.command()
@click.option("--board" , "-b", default="v2.1", type=click.Choice(['v2.0', 'v2.1']), help="Board revision.", show_default=True)
@click.option("--device", "-d", default="/dev/input/event0", help="Input device to use.", show_default=True)
@click.option("--port"  , "-p", default='0', type=click.Choice(['0', '1']), help="Board/Atari port to connect the mouse to.", show_default=True)
@click.option("--speed" , "-s", default=2, help="Mouse speed divider (more = slower).", show_default=True)
@click.option("--debug/--no-debug", help="Display debugging information.", show_default=True)
def main(board, device, port, speed, debug):
    """Send mouse events to an Atari ST connected to the RetroDevEm
    board.  Usage example: atarist-mouse --board v2.0 --device
    /dev/input/event0 --port 0 --speed 4

    """
    if debug:
        logging.basicConfig(level=logging.DEBUG)
    try:
        sm = StMouse(board_version=board, port=int(port), xy_scale=speed)
        process_input_events(device, sm)
    except BadPinFactory as e:
        logger.error(f"Failed to initialize GPIO pins: {e}")
        logger.error( "This program expects to be running on a Raspberry Pi. Is that the case ?")

if __name__ == "__main__":
    main()



import fcntl
import os
import struct

# Constants for the event structure
# https://www.kernel.org/doc/html/latest/input/input.html
# https://docs.python.org/3/library/struct.html
EVENT_FORMAT = 'llHHi' # long, long, uint16_t, uint16_t, uint32_t
EVENT_SIZE = struct.calcsize(EVENT_FORMAT)

# Obtained using the evtest CLI
# Events types
EV_KEY = 1
EV_REL = 2
EV_ABS = 3
# Events codes
# Mouse events codes
REL_X = 0 # Mouse x move
REL_Y = 1 # Mouse y move
BTN_LEFT = 272 # Mouse left button
BTN_RIGHT = 273 # Mouse right button
# Gamepad events codes
ABS_HAT0X = 16 # Gamepad directional arrows left (-1) right (+1)
ABS_HAT0Y = 17 # Gamepad directional arrows down (-1) up (+1)
BTN_SOUTH = 304 # X button off (-1) on (+1)
ABS_X     = 0 # Gamepad analog left-right axis [0, 255]
ABS_Y     = 1 # Gamepad analog up-down axis [0, 255]

# USB mouse events location
USB_DEVICE = "/dev/input/event0"

class InputDevice:
    def __init__(self, device="/dev/input/event0", blocking=True):
        """device the path to the input device."""
        self.device = open(device, 'rb', buffering=0)
        if not blocking:
            fd = self.device.fileno()
            flag = fcntl.fcntl(fd, fcntl.F_GETFL)
            fcntl.fcntl(fd, fcntl.F_SETFL, flag | os.O_NONBLOCK)

    def get_event(self):
        """Retrieve a single event.  Either blocks until an event is
        available (in blocking mode), or returns None if no event were
        found while file opened in non-blocking mode.

        """
        event_data = self.device.read(EVENT_SIZE)
        if event_data:
            return struct.unpack(EVENT_FORMAT, event_data)



#!/usr/bin/env python3
"""Used for testing purpose"""

import time
from gpiozero import LED

TICK_PERIOD = 1/1000 # 1 KHz period

def main():
    led = LED(18)
    
    next_tick = time.monotonic()
    while True:
        next_tick += TICK_PERIOD/2
        if led.is_active: led.off()
        else: led.on()
        try:
            time.sleep( next_tick - time.monotonic() )
        except ValueError:
            pass

main()
